# Settings

# Rods
# Number of rods
set N 80
# Particles per rod (has to be an odd number!)
set particlesPerRod 5 

# Volume fraction
set rho 0.5



# Thermostat
# Temperature
set T 1
# Friction
set gamma 0.5

# Lennard-Jones Potentials
set ljSigma 1
set ljCut [expr $ljSigma *pow(2,1./6.)]
set ljEpsilon 5

# Coulomb interaction
# Charge per particle
set Q 1
# Bjerum length
set bjerum 20
# p3m accuracy
set p3mAcc 1E-3
# Skin
set skin 0.4
# Number of cells for domain decomposition
set cells [expr 8*8*8]
# Time step
set dt 0.0003


# How long to run
# Number of visualization updateds
set intStepsLarge 10000 
# Integration steps between updates
set intStepsSmall 300

# Calculate some dependent parameters
# Minimal global cutoff (from center of rod to end of rodd)
set minGlobalCut [expr ($particlesPerRod -1)/2 *$ljSigma +0.01]

# Box length
set box_l [expr pow(1./6.*3.142 *pow($ljSigma,3)*$N*$particlesPerRod/$rho,1./3.)]
puts "$box_l"



# SEtup the integrator and the interactions
proc setupMD {} {
 # Import global variables
 global box_l
 global skin
 global cells
 global minGlobalCut
 global T
 global gamma
 global dt

 # Box length
 setmd box_l $box_l $box_l $box_l
 
 # Integrator
 setmd time_step $dt
 setmd skin $skin
 setmd max_num_cells $cells
 setmd min_global_cut $minGlobalCut

 # Thermostat
 thermostat langevin $T $gamma

}

# Setup the interactions
proc setupLJInteraction {} {
 # Import global vars
 global ljEpsilon
 global ljSigma
 global ljCut

 # Setup interaction
 inter 0 0 lennard-jones $ljEpsilon $ljSigma $ljCut auto
}

# Coulmb interaction
proc setupCoulomb {} {
 # IMport global vars
 global p3mAcc
 global bjerum

 # SEtup interaciton
 inter coulomb $bjerum p3m tunev2 accuracy $p3mAcc
}

#A bonded interaction is used to visualize the rods as lines
proc setupFakeBondedIA {} {
 inter 0 virtual
}


# Warmup with force cap
proc warmupWithForceCap {} {
 # Integrate some steps with the force capped
 # The cap is increased in several steps
 for {set cap 5} {$cap <400} {incr cap 5} {
  inter ljforcecap $cap
  integrate 1000
  puts [analyze mindist]
 }
 inter ljforcecap 0
}

# Lace a rod ad a random position with a random orientation
proc placeRandomRod {} {
 # Import global vars
 global particlesPerRod
 global Q
 global box_l
 global ljSigma
 global Q

 # Get a random point on the unit sphere, which will be the director
 # of the rod
 set costheta [expr 2*[expr [t_random]] - 1]
 set sintheta [expr sin(acos($costheta))]
 set phi [expr 2*3.1415926536*[expr [t_random]]] 
 set dx [expr $sintheta*cos($phi)]
 set dy [expr $sintheta*sin($phi)]
 set dz [expr $costheta]

 # Get a random position for the central particle fo the rod
 set cx [expr $box_l *[t_random]]
 set cy [expr $box_l *[t_random]]
 set cz [expr $box_l *[t_random]]

 # Place central particle of the rod
 # This has to be there first, because it is the center of mass
 # of the rigid body. The remaining particles of the rod are "related" to it
 set particleIdCenter [expr [setmd max_part]+1]
 part $particleIdCenter pos $cx $cy $cz q $Q 

 # Now place the remaining particles or the rod
 for {set i [expr -($particlesPerRod-1)/2]} {$i <=($particlesPerRod-1)/2} {incr i} {
  # If this is the central particles, which was already placed, continue
  if {$i==0} { continue } 

  # Calc the particle position
  set x [expr $cx +$i*$ljSigma*$dx]
  set y [expr $cy +$i*$ljSigma*$dy]
  set z [expr $cz +$i*$ljSigma*$dz]

  # Place the particle
  set particleId [expr [setmd max_part]+1]
  part $particleId pos $x $y $z 
  # Make the particle a virtual site and "relate" it to the central particle
  # of the rod
  part $particleId virtual 1 vs_auto_relate_to $particleIdCenter q $Q
 }

 # For visualization, we add a bond between the first and last particle of the rod
 set firstPID [expr $particleIdCenter+1]
 set lastPID [expr $particleIdCenter+$particlesPerRod-1]
 part $firstPID bond 0 $lastPID
}


# Setup the vmd visualization
proc setupVMD {} {
 # prelace the 1 by 0 and call vmd manually  with like
 # vmd -e rods.vmd 
 # to visualize as rods
 puts "[prepare_vmd_connection "rods" 20 1]"
 imd position
} 


# SEtup general md params and integrator
setupMD

# Setup the potentials
setupLJInteraction
setupFakeBondedIA 

# Place the rods
for {set i 0} {$i < $N} {incr i} {
 placeRandomRod
}


# Warmup with force cap to make sure, rods don't overlap
warmupWithForceCap

# Setup coulomb interaction
setupCoulomb

# Setup vmd connection
setupVMD

# Main integration
for {set i 0} {$i < $intStepsLarge} { incr i} {
 integrate $intStepsSmall
 imd positions
 puts "Step $i"
}


  
 
